@@ -24,6 +24,7 @@ import os
 import re
 import sys
 
+from os_vif.internal.command import ip as ip_lib
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
@@ -107,12 +108,17 @@ def device_exists(device):
     return os.path.exists('/sys/class/net/%s' % device)
 
 
+def interface_in_bridge(bridge, device):
+    """Check if an ethernet device belongs to a Linux Bridge."""
+    return os.path.exists('/sys/class/net/%(bridge)s/brif/%(device)s' %
+                          {'bridge': bridge, 'device': device})
+
+
 def _delete_net_dev(dev):
     """Delete a network device only if it exists."""
     if device_exists(dev):
         try:
-            processutils.execute('ip', 'link', 'delete', dev,
-                                 check_exit_code=[0, 2, 254])
+            ip_lib.delete(dev, check_exit_code=[0, 2, 254])
             LOG.debug("Net device removed: '%s'", dev)
         except processutils.ProcessExecutionError:
             with excutils.save_and_reraise_exception():
@@ -127,11 +133,10 @@ def create_veth_pair(dev1_name, dev2_name, mtu):
     for dev in [dev1_name, dev2_name]:
         _delete_net_dev(dev)
 
-    processutils.execute('ip', 'link', 'add', dev1_name,
-                         'type', 'veth', 'peer', 'name', dev2_name)
+    ip_lib.add(dev1_name, 'veth', peer=dev2_name)
     for dev in [dev1_name, dev2_name]:
-        processutils.execute('ip', 'link', 'set', dev, 'up')
-        processutils.execute('ip', 'link', 'set', dev, 'promisc', 'on')
+        ip_lib.set(dev, state='up')
+        ip_lib.set(dev, promisc='on')
         _update_device_mtu(dev, mtu)
 
 
@@ -153,6 +158,7 @@ def ensure_bridge(bridge):
         processutils.execute('brctl', 'addbr', bridge)
         processutils.execute('brctl', 'setfd', bridge, 0)
         processutils.execute('brctl', 'stp', bridge, 'off')
+        processutils.execute('brctl', 'setageing', bridge, 0)
         syspath = '/sys/class/net/%s/bridge/multicast_snooping'
         syspath = syspath % bridge
         processutils.execute('tee', syspath, process_input='0',
@@ -171,8 +177,10 @@ def ensure_bridge(bridge):
 @privsep.vif_plug.entrypoint
 def delete_bridge(bridge, dev):
     if device_exists(bridge):
-        processutils.execute('brctl', 'delif', bridge, dev)
-        processutils.execute('ip', 'link', 'set', bridge, 'down')
+        if interface_in_bridge(bridge, dev):
+            processutils.execute('brctl', 'delif', bridge, dev)
+
+        ip_lib.set(bridge, state='down')
         processutils.execute('brctl', 'delbr', bridge)
 
 
@@ -205,14 +213,12 @@ def _update_device_mtu(dev, mtu, interface_type=None, timeout=120):
 @privsep.vif_plug.entrypoint
 def _set_device_mtu(dev, mtu):
     """Set the device MTU."""
-    processutils.execute('ip', 'link', 'set', dev, 'mtu', mtu,
-                         check_exit_code=[0, 2, 254])
+    ip_lib.set(dev, mtu=mtu, check_exit_code=[0, 2, 254])
 
 
 @privsep.vif_plug.entrypoint
 def set_interface_state(interface_name, port_state):
-    processutils.execute('ip', 'link', 'set', interface_name, port_state,
-                         check_exit_code=[0, 2, 254])
+    ip_lib.set(interface_name, state=port_state, check_exit_code=[0, 2, 254])
 
 
 @privsep.vif_plug.entrypoint
