@@ -56,7 +56,12 @@ class PluginTest(testtools.TestCase):
             mtu=1234)
 
         self.profile_ovs = objects.vif.VIFPortProfileOpenVSwitch(
-            interface_id='e65867e0-9340-4a7f-a256-09af6eb7a3aa')
+            interface_id='e65867e0-9340-4a7f-a256-09af6eb7a3aa',
+            datapath_type='netdev')
+
+        self.profile_ovs_no_datatype = objects.vif.VIFPortProfileOpenVSwitch(
+            interface_id='e65867e0-9340-4a7f-a256-09af6eb7a3aa',
+            datapath_type='')
 
         self.vif_ovs_hybrid = objects.vif.VIFBridge(
             id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
@@ -64,7 +69,7 @@ class PluginTest(testtools.TestCase):
             network=self.network_ovs,
             dev_name='tap-xxx-yyy-zzz',
             bridge_name="qbrvif-xxx-yyy",
-            port_profile=self.profile_ovs)
+            port_profile=self.profile_ovs_no_datatype)
 
         self.vif_ovs = objects.vif.VIFOpenVSwitch(
             id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
@@ -102,9 +107,19 @@ class PluginTest(testtools.TestCase):
             name='demo',
             uuid='f0000000-0000-0000-0000-000000000001')
 
+    def test__get_vif_datapath_type(self):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        dp_type = plugin._get_vif_datapath_type(
+            self.vif_ovs, datapath=constants.OVS_DATAPATH_SYSTEM)
+        self.assertEqual(self.profile_ovs.datapath_type, dp_type)
+
+        dp_type = plugin._get_vif_datapath_type(
+            self.vif_ovs_hybrid, datapath=constants.OVS_DATAPATH_SYSTEM)
+        self.assertEqual(constants.OVS_DATAPATH_SYSTEM, dp_type)
+
     @mock.patch.object(linux_net, 'create_ovs_vif_port')
     def test_create_vif_port(self, mock_create_ovs_vif_port):
-        plugin = ovs.OvsPlugin.load('ovs')
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin._create_vif_port(
             self.vif_ovs, mock.sentinel.vif_name, self.instance,
             interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
@@ -119,7 +134,7 @@ class PluginTest(testtools.TestCase):
     @mock.patch.object(linux_net, 'create_ovs_vif_port')
     def test_create_vif_port_mtu_in_model(self, mock_create_ovs_vif_port):
         self.vif_ovs.network = self.network_ovs_mtu
-        plugin = ovs.OvsPlugin.load('ovs')
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin._create_vif_port(
             self.vif_ovs, mock.sentinel.vif_name, self.instance,
             interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
@@ -136,12 +151,12 @@ class PluginTest(testtools.TestCase):
     def test_plug_ovs(self, ensure_ovs_bridge, mock_sys):
         mock_sys.platform = 'linux'
         plug_bridge_mock = mock.Mock()
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin._plug_bridge = plug_bridge_mock
         plugin.plug(self.vif_ovs, self.instance)
-        plug_bridge_mock.assert_not_called()
-        ensure_ovs_bridge.assert_called_once_with(
-            self.vif_ovs.network.bridge, constants.OVS_DATAPATH_SYSTEM)
+        dp_type = ovs.OvsPlugin._get_vif_datapath_type(self.vif_ovs)
+        ensure_ovs_bridge.assert_called_once_with(self.vif_ovs.network.bridge,
+                                                  dp_type)
 
     @mock.patch.object(linux_net, 'set_interface_state')
     @mock.patch.object(linux_net, 'ensure_ovs_bridge')
@@ -158,6 +173,7 @@ class PluginTest(testtools.TestCase):
                              add_bridge_port, _create_vif_port,
                              _update_vif_port, ensure_ovs_bridge,
                              set_interface_state):
+        dp_type = ovs.OvsPlugin._get_vif_datapath_type(self.vif_ovs_hybrid)
         calls = {
             'device_exists': [mock.call('qvob679325f-ca')],
             'create_veth_pair': [mock.call('qvbb679325f-ca',
@@ -176,15 +192,14 @@ class PluginTest(testtools.TestCase):
             '_create_vif_port': [mock.call(self.vif_ovs_hybrid,
                                            'qvob679325f-ca',
                                            self.instance)],
-            'ensure_ovs_bridge': [mock.call('br0',
-                                            constants.OVS_DATAPATH_SYSTEM)]
+            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
         }
 
         # plugging new devices should result in devices being created
 
         device_exists.return_value = False
         mock_sys.platform = 'linux'
-        plugin = ovs.OvsPlugin.load('ovs')
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_ovs_hybrid, self.instance)
         ensure_bridge.assert_has_calls(calls['ensure_bridge'])
         device_exists.assert_has_calls(calls['device_exists'])
@@ -216,15 +231,15 @@ class PluginTest(testtools.TestCase):
     @mock.patch.object(ovs, 'sys')
     def _check_plug_ovs_windows(self, vif, mock_sys, device_exists,
                                 _create_vif_port, ensure_ovs_bridge):
+        dp_type = ovs.OvsPlugin._get_vif_datapath_type(vif)
         calls = {
             'device_exists': [mock.call(vif.id)],
             '_create_vif_port': [mock.call(vif, vif.id, self.instance)],
-            'ensure_ovs_bridge': [mock.call('br0',
-                                            constants.OVS_DATAPATH_SYSTEM)]
+            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
         }
 
         mock_sys.platform = constants.PLATFORM_WIN32
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(vif, self.instance)
         device_exists.assert_has_calls(calls['device_exists'])
         _create_vif_port.assert_has_calls(calls['_create_vif_port'])
@@ -238,7 +253,7 @@ class PluginTest(testtools.TestCase):
 
     def test_unplug_ovs(self):
         unplug_bridge_mock = mock.Mock()
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin._unplug_bridge = unplug_bridge_mock
         plugin.unplug(self.vif_ovs, self.instance)
         unplug_bridge_mock.assert_not_called()
@@ -254,7 +269,7 @@ class PluginTest(testtools.TestCase):
                                     timeout=120)]
         }
         mock_sys.platform = 'linux'
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_ovs_hybrid, self.instance)
         delete_bridge.assert_has_calls(calls['delete_bridge'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
@@ -263,7 +278,7 @@ class PluginTest(testtools.TestCase):
     @mock.patch.object(ovs, 'sys')
     def _check_unplug_ovs_windows(self, vif, mock_sys, delete_ovs_vif_port):
         mock_sys.platform = constants.PLATFORM_WIN32
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(vif, self.instance)
         delete_ovs_vif_port.assert_called_once_with('br0', vif.id, timeout=120)
 
@@ -276,17 +291,17 @@ class PluginTest(testtools.TestCase):
     @mock.patch.object(linux_net, 'ensure_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_create_vif_port')
     def test_plug_ovs_vhostuser(self, _create_vif_port, ensure_ovs_bridge):
+        dp_type = ovs.OvsPlugin._get_vif_datapath_type(self.vif_vhostuser)
         calls = {
 
             '_create_vif_port': [mock.call(
                                  self.vif_vhostuser, 'vhub679325f-ca',
                                  self.instance,
                                  interface_type='dpdkvhostuser')],
-            'ensure_ovs_bridge': [mock.call('br0',
-                                            constants.OVS_DATAPATH_NETDEV)]
+            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
         }
 
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_vhostuser, self.instance)
         _create_vif_port.assert_has_calls(calls['_create_vif_port'])
         ensure_ovs_bridge.assert_has_calls(calls['ensure_ovs_bridge'])
@@ -295,6 +310,8 @@ class PluginTest(testtools.TestCase):
     @mock.patch.object(linux_net, 'create_ovs_vif_port')
     def test_plug_ovs_vhostuser_client(self, create_ovs_vif_port,
                                        ensure_ovs_bridge):
+        dp_type = ovs.OvsPlugin._get_vif_datapath_type(
+            self.vif_vhostuser_client)
         calls = {
             'create_ovs_vif_port': [
                  mock.call(
@@ -305,11 +322,10 @@ class PluginTest(testtools.TestCase):
                      1500, interface_type='dpdkvhostuserclient',
                      vhost_server_path='/var/run/openvswitch/vhub679325f-ca',
                      timeout=120)],
-            'ensure_ovs_bridge': [mock.call('br0',
-                                            constants.OVS_DATAPATH_NETDEV)]
+            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
         }
 
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_vhostuser_client, self.instance)
         create_ovs_vif_port.assert_has_calls(calls['create_ovs_vif_port'])
         ensure_ovs_bridge.assert_has_calls(calls['ensure_ovs_bridge'])
@@ -320,7 +336,7 @@ class PluginTest(testtools.TestCase):
             'delete_ovs_vif_port': [mock.call('br0', 'vhub679325f-ca',
                                     timeout=120)]
         }
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_vhostuser, self.instance)
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
 
@@ -355,7 +371,7 @@ class PluginTest(testtools.TestCase):
                                  self.instance)]
         }
 
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_ovs_vf_passthrough, self.instance)
         ensure_ovs_bridge.assert_has_calls(calls['ensure_ovs_bridge'])
         get_ifname_by_pci_address.assert_has_calls(
@@ -392,7 +408,7 @@ class PluginTest(testtools.TestCase):
         get_ifname_by_pci_address.return_value = 'eth0'
         get_vf_num_by_pci_address.return_value = '2'
         get_representor_port.return_value = 'eth0_2'
-        plugin = ovs.OvsPlugin.load("ovs")
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_ovs_vf_passthrough, self.instance)
         get_ifname_by_pci_address.assert_has_calls(
             calls['get_ifname_by_pci_address'])
